@@ -12,6 +12,7 @@ import java.util.Map;
 import org.gcube.portlets.user.td.widgetcommonevent.client.event.GridHeaderColumnMenuItemEvent;
 import org.gcube.portlets.user.td.widgetcommonevent.client.type.GridHeaderColumnMenuItemType;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.CellData;
+import org.gcube.portlets.user.td.widgetcommonevent.shared.grid.model.RowRaw;
 import org.gcube.portlets.user.tdwx.client.config.GridAndCellSelectionModel;
 import org.gcube.portlets.user.tdwx.client.config.Row;
 import org.gcube.portlets.user.tdwx.client.config.TableViewConfig;
@@ -23,6 +24,8 @@ import org.gcube.portlets.user.tdwx.client.event.OpenTableEvent;
 import org.gcube.portlets.user.tdwx.client.event.OpenTableEventHandler;
 import org.gcube.portlets.user.tdwx.client.event.TableReadyEvent;
 import org.gcube.portlets.user.tdwx.client.event.TableReadyEventHandler;
+import org.gcube.portlets.user.tdwx.client.filter.ExtendedGridFilters;
+import org.gcube.portlets.user.tdwx.client.filter.FiltersGenerator;
 import org.gcube.portlets.user.tdwx.client.model.grid.DataRowColumnConfig;
 import org.gcube.portlets.user.tdwx.client.model.grid.DataRowModelKeyProvider;
 import org.gcube.portlets.user.tdwx.client.model.grid.DataRowPagingReader;
@@ -51,8 +54,9 @@ import com.sencha.gxt.core.client.util.Margins;
 import com.sencha.gxt.data.client.loader.HttpProxy;
 import com.sencha.gxt.data.shared.ListStore;
 import com.sencha.gxt.data.shared.event.StoreUpdateEvent;
-import com.sencha.gxt.data.shared.loader.PagingLoadConfig;
-import com.sencha.gxt.data.shared.loader.PagingLoadConfigBean;
+import com.sencha.gxt.data.shared.loader.FilterPagingLoadConfig;
+import com.sencha.gxt.data.shared.loader.FilterPagingLoadConfigBean;
+import com.sencha.gxt.data.shared.loader.LoadResultListStoreBinding;
 import com.sencha.gxt.data.shared.loader.PagingLoadResult;
 import com.sencha.gxt.data.shared.loader.PagingLoader;
 import com.sencha.gxt.widget.core.client.ContentPanel;
@@ -63,14 +67,15 @@ import com.sencha.gxt.widget.core.client.event.HeaderContextMenuEvent.HeaderCont
 import com.sencha.gxt.widget.core.client.event.HeaderMouseDownEvent;
 import com.sencha.gxt.widget.core.client.event.HeaderMouseDownEvent.HeaderMouseDownHandler;
 import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
-import com.sencha.gxt.widget.core.client.grid.ColumnHeader;
 import com.sencha.gxt.widget.core.client.grid.ColumnModel;
 import com.sencha.gxt.widget.core.client.grid.Grid;
 import com.sencha.gxt.widget.core.client.grid.LiveGridView;
 import com.sencha.gxt.widget.core.client.grid.LiveToolItem;
+import com.sencha.gxt.widget.core.client.grid.filters.Filter;
 import com.sencha.gxt.widget.core.client.menu.Item;
 import com.sencha.gxt.widget.core.client.menu.Menu;
 import com.sencha.gxt.widget.core.client.menu.MenuItem;
+import com.sencha.gxt.widget.core.client.menu.SeparatorMenuItem;
 import com.sencha.gxt.widget.core.client.selection.CellSelection;
 import com.sencha.gxt.widget.core.client.toolbar.ToolBar;
 
@@ -94,15 +99,19 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 	protected ListStore<DataRow> store;
 	protected Grid<DataRow> grid;
+	protected ToolBar toolBar;
 	// protected GridInlineEditing<DataRow> editing;
 	protected TableDefinition tableDefinition;
+
 	protected VerticalLayoutContainer container;
 
 	protected ExtendedLiveGridView liveGridView;
 
 	protected DataRowPagingReader reader;
 
-	protected PagingLoader<PagingLoadConfig, PagingLoadResult<DataRow>> loader;
+	protected PagingLoader<FilterPagingLoadConfig, PagingLoadResult<DataRow>> loader;
+
+	protected ExtendedGridFilters<DataRow> gridFilters;
 
 	protected TableViewConfig tableViewConfig;
 
@@ -165,6 +174,15 @@ public class TabularDataXGridPanel extends ContentPanel {
 		this.contextMenu = contextMenu;
 	}
 
+	/**
+	 * Return table definition
+	 * 
+	 * @return
+	 */
+	public TableDefinition getTableDefinition() {
+		return tableDefinition;
+	}
+
 	/**
 	 * Returns the selected row.
 	 * 
@@ -221,6 +239,65 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 		return null;
 	}
+	
+	
+	/**
+	 * 
+	 * 
+	 * @return Selected Row as RowRaw
+	 */
+	public RowRaw getSelectedRowAsRaw() {
+		if (grid != null && grid.getSelectionModel() != null) {
+			List<DataRow> dataRows = grid.getSelectionModel()
+					.getSelectedItems();
+			List<Row> rows = new ArrayList<Row>();
+			for (DataRow dataRow : dataRows) {
+				if (dataRow != null) {
+					rows.add(new Row(dataRow, keys));
+				}
+			}
+
+			RowRaw r = null;
+			
+			if (rows.size() > 0) {
+				HashMap<String, String> map = new HashMap<String, String>();
+				ColumnModel<DataRow> columnModel = grid.getColumnModel();
+				List<ColumnConfig<DataRow, ?>> columns = columnModel
+						.getColumns();
+				Row rowFirst = rows.get(0);
+				DataRowColumnConfig<?> columnDataRow = null;
+				String rowId=null;
+				for (ColumnConfig<DataRow, ?> col : columns) {
+					columnDataRow = (DataRowColumnConfig<?>) col;
+					ColumnDefinition colDef = columnDataRow.getDefinition();
+					if (colDef != null) {
+						String value;
+						if (colDef.getColumnDataType()
+								.compareTo("Date") == 0) {
+							value = rowFirst.getFieldAsDate(colDef
+									.getColumnLocalId());
+						} else {
+							value = rowFirst.getFieldAsText(colDef
+									.getColumnLocalId());
+						}
+						map.put(colDef.getColumnLocalId(), value);
+						ColumnType ctype = colDef.getType();
+						if (ctype == ColumnType.COLUMNID) {
+							rowId=value;
+						}
+					
+					}
+				}
+				if(rowId!=null && !rowId.isEmpty() && map.size()>0){
+					r=new RowRaw(rowId,map);
+				}
+
+			}
+			return r;
+		}
+
+		return null;
+	}
 
 	/**
 	 * Retrieve selected cell value
@@ -261,11 +338,18 @@ public class TabularDataXGridPanel extends ContentPanel {
 							.getCell());
 					DataRowColumnConfig<?> cSelected = (DataRowColumnConfig<?>) colSelected;
 					ColumnDefinition cdSelected = cSelected.getDefinition();
-					String cellValue = row
-							.getFieldAsText(cdSelected.getLabel());
+
+					String cellValue;
+					if (cdSelected.getColumnDataType().compareTo("Date") == 0) {
+						cellValue = row.getFieldAsDate(cdSelected
+								.getColumnLocalId());
+					} else {
+						cellValue = row.getFieldAsText(cdSelected
+								.getColumnLocalId());
+					}
 
 					String columnIdValue = row.getFieldAsText(columnIdSelected
-							.getLabel());
+							.getColumnLocalId());
 
 					CellData tdCell = new CellData(cellValue,
 							cdSelected.getId(), cdSelected.getColumnLocalId(),
@@ -294,20 +378,20 @@ public class TabularDataXGridPanel extends ContentPanel {
 	 * @param columnLocalId
 	 */
 	public void setVisibleOnlyColumn(String columnLocalId) {
-		Log.debug("setVisibleOnlyColumn: "+columnLocalId);
-		visibleOnlyColumn=columnLocalId;
-		
+		Log.debug("setVisibleOnlyColumn: " + columnLocalId);
+		visibleOnlyColumn = columnLocalId;
+
 	}
-	
+
 	/**
 	 * 
 	 * @param columnModel
 	 * @return
 	 */
-	protected ColumnModel<DataRow> checkOnlyColumn(ColumnModel<DataRow> columnModel){	
-		if (visibleOnlyColumn!=null && !visibleOnlyColumn.isEmpty()) {
+	protected ColumnModel<DataRow> checkOnlyColumn(
+			ColumnModel<DataRow> columnModel) {
+		if (visibleOnlyColumn != null && !visibleOnlyColumn.isEmpty()) {
 			Log.debug("setVisibleOnlyColumn: Grid not null");
-			
 
 			List<ColumnConfig<DataRow, ?>> columns = columnModel.getColumns();
 
@@ -329,7 +413,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 				}
 			}
 			columnModel = new ColumnModel<DataRow>(columnsNew);
-			
+
 		} else {
 			Log.debug("visibleOnlyColumn null");
 		}
@@ -374,7 +458,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 				ArrayList<String> rows = new ArrayList<String>();
 				if (rowsSelected != null) {
 					for (Row row : rowsSelected) {
-						rowS = row.getFieldAsText(cd.getLabel());
+						rowS = row.getFieldAsText(cd.getColumnLocalId());
 						Log.debug("Selected Row:" + rowS);
 						rows.add(rowS);
 					}
@@ -421,7 +505,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 				ArrayList<String> rows = new ArrayList<String>();
 				if (rowsSelected != null) {
 					for (Row row : rowsSelected) {
-						rowS = row.getFieldAsText(cd.getLabel());
+						rowS = row.getFieldAsText(cd.getColumnLocalId());
 						Log.debug("Selected Row:" + rowS);
 						rows.add(rowS);
 					}
@@ -477,7 +561,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 		this.currentTableDefinition = definition;
 		keys = new HashMap<String, ColumnKey>();
 		for (ColumnDefinition column : definition.getColumnsAsList())
-			keys.put(column.getLabel(), column.getKey());
+			keys.put(column.getColumnLocalId(), column.getKey());
 		unmask();
 	}
 
@@ -489,33 +573,35 @@ public class TabularDataXGridPanel extends ContentPanel {
 		container.clear();
 		unmask();
 	}
-	
+
 	/**
 	 * 
 	 * @param handler
 	 */
 	protected void addHeaderMouseDownHandler(HeaderMouseDownHandler handler) {
 		if (grid != null) {
-			
+
 			if (handler == null) {
 				handler = new HeaderMouseDownHandler() {
 
 					@Override
 					public void onHeaderMouseDown(HeaderMouseDownEvent event) {
-						Log.debug("HeaderMouseDownEvent :"+event.toDebugString());
+						Log.debug("HeaderMouseDownEvent :"
+								+ event.toDebugString());
 						if (event.getEvent().getButton() == NativeEvent.BUTTON_RIGHT) {
 							event.getEvent().stopPropagation();
 						} else {
-							
+
 						}
 
 					}
 				};
-			};
+			}
+			;
 			grid.addHeaderMouseDownHandler(handler);
 		}
 	}
-	
+
 	/**
 	 * Define the grid
 	 * 
@@ -557,27 +643,52 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 		ColumnModel<DataRow> columnModel = new ColumnModel<DataRow>(
 				columnsConfig);
-		
-		columnModel =checkOnlyColumn(columnModel);
-		
+
+		columnModel = checkOnlyColumn(columnModel);
+
 		if (grid == null) {
+
 			reader = new DataRowPagingReader(tableDefinition);
 
 			String path = GWT.getModuleBaseURL() + "tdwxdata";
-			RequestBuilder builder = new RequestBuilder(RequestBuilder.GET,
+			RequestBuilder builder = new RequestBuilder(RequestBuilder.POST,
 					path);
 			builder.setHeader(ServletParameters.TD_SESSION_ID,
 					String.valueOf(tdSessionId));
-			HttpProxy<PagingLoadConfig> proxy = new HttpProxy<PagingLoadConfig>(
+			// builder.setHeader("Content-Type",
+			// "application/x-www-form-urlencoded");
+
+			HttpProxy<FilterPagingLoadConfig> proxy = new HttpProxy<FilterPagingLoadConfig>(
 					builder);
 			proxy.setWriter(new PagingLoadUrlEncoder());
 
-			loader = new PagingLoader<PagingLoadConfig, PagingLoadResult<DataRow>>(
-					proxy, reader);
+			loader = new PagingLoader<FilterPagingLoadConfig, PagingLoadResult<DataRow>>(
+					proxy, reader) {
+				@Override
+				protected FilterPagingLoadConfig newLoadConfig() {
+					return new FilterPagingLoadConfigBean();
+				}
+			};
+
+			/*
+			 * HttpProxy<PagingLoadConfig> proxy = new
+			 * HttpProxy<PagingLoadConfig>( builder); proxy.setWriter(new
+			 * PagingLoadUrlEncoder());
+			 * 
+			 * loader = new PagingLoader<PagingLoadConfig,
+			 * PagingLoadResult<DataRow>>( proxy, reader) {
+			 * 
+			 * @Override protected PagingLoadConfig newLoadConfig() { return new
+			 * PagingLoadConfigBean(); } };
+			 */
+
 			loader.setRemoteSort(true);
 
+			loader.addLoadHandler(new LoadResultListStoreBinding<FilterPagingLoadConfig, DataRow, PagingLoadResult<DataRow>>(
+					store));
+
 			liveGridView = new ExtendedLiveGridView();
-			
+
 			// liveGridView.setRowHeight(19);
 
 			grid = new Grid<DataRow>(store, columnModel);
@@ -587,9 +698,7 @@ public class TabularDataXGridPanel extends ContentPanel {
 			grid.setLoadMask(true);
 			grid.setLoader(loader);
 			grid.setView(liveGridView);
-			
-			
-			
+
 			// TODO disabled editing rows
 			/*
 			 * editing = new GridInlineEditing<DataRow>(grid);
@@ -601,9 +710,22 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 			// numberer.initPlugin(grid);
 
+			// Filter
+			ArrayList<Filter<DataRow, ?>> filters = FiltersGenerator
+					.generate(columnModel);
+			gridFilters = new ExtendedGridFilters<DataRow>(loader);
+
+			gridFilters.initPlugin(grid);
+			gridFilters.setLocal(false);
+			Log.debug("Filters: " + filters.size());
+			for (Filter<DataRow, ?> filterGeneric : filters) {
+				gridFilters.addFilter(filterGeneric);
+			}
+
+			//
 			container.add(grid, new VerticalLayoutData(1, 1, new Margins(0)));
 
-			ToolBar toolBar = new ToolBar();
+			toolBar = new ToolBar();
 			toolBar.add(new LiveToolItem(grid));
 			toolBar.addStyleName(ThemeStyles.getStyle().borderTop());
 			toolBar.getElement().getStyle().setProperty("borderBottom", "none");
@@ -613,13 +735,6 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 			container.forceLayout();
 
-			Scheduler.get().scheduleDeferred(new ScheduledCommand() {
-
-				public void execute() {
-					container.forceLayout();
-				}
-			});
-
 			loader.load(0, liveGridView.getCacheSize());
 
 			if (contextMenu != null) {
@@ -635,22 +750,32 @@ public class TabularDataXGridPanel extends ContentPanel {
 			// TODO we need to manually reset the sort
 			loader.clearSortInfo();
 			reader.setDefinition(tableDefinition);
+
 			grid.reconfigure(store, columnModel);
 
+			ArrayList<Filter<DataRow, ?>> filters = FiltersGenerator
+					.generate(columnModel);
+			gridFilters.removeAll();
+
+			Log.debug("Filters: " + filters.size());
+			for (Filter<DataRow, ?> filterGeneric : filters) {
+				gridFilters.addFilter(filterGeneric);
+			}
+
 			// TODO Disabled editing rows
-			/*
-			 * editing = new GridInlineEditing<DataRow>(grid);
-			 * editing.setClicksToEdit(ClicksToEdit.TWO);
-			 * 
-			 * for (ColumnConfig<DataRow, ?> c : columnsConfig) {
-			 * ColumnConfigGenerator.setEditor(editing, c); }
-			 */
+			//
+			// editing = new GridInlineEditing<DataRow>(grid);
+			// editing.setClicksToEdit(ClicksToEdit.TWO);
+			//
+			// for (ColumnConfig<DataRow, ?> c : columnsConfig) {
+			// ColumnConfigGenerator.setEditor(editing, c); }
+			//
 
 			// TODO workaround: the loader is called only if the scroll bar is
 			// in the middle
-			if (loader.getOffset() == 0)
-				loader.load(new PagingLoadConfigBean(0, liveGridView
-						.getCacheSize()));
+			// if (loader.getOffset() == 0)
+			// loader.load();
+			loader.load(0, liveGridView.getCacheSize());
 		}
 
 		if (tableViewConfig != null) {
@@ -662,7 +787,16 @@ public class TabularDataXGridPanel extends ContentPanel {
 		} else {
 			grid.getView().setViewConfig(null);
 		}
-		
+
+		container.forceLayout();
+
+		Scheduler.get().scheduleDeferred(new ScheduledCommand() {
+
+			public void execute() {
+				container.forceLayout();
+			}
+		});
+
 		eventBus.fireEvent(new GridReadyEvent());
 	}
 
@@ -716,6 +850,8 @@ public class TabularDataXGridPanel extends ContentPanel {
 				menu.addSelectionHandler(handlerHeaderContextMenu);
 
 				Log.debug("Adding Items to menu");
+				SeparatorMenuItem separatorItem = new SeparatorMenuItem();
+				menu.add(separatorItem);
 				for (MenuItem m : headerColumnMenuItems) {
 					menu.add(m);
 				}
@@ -776,10 +912,13 @@ public class TabularDataXGridPanel extends ContentPanel {
 
 	protected class ExtendedLiveGridView extends LiveGridView<DataRow> {
 		// TODO bug in gxt3 3.0.0 fixed in future
-		/*
-		 * @Override public void refresh(boolean headerToo) {
-		 * preventScrollToTopOnRefresh = true; super.refresh(headerToo); }
-		 */
+
+		@Override
+		public void refresh(boolean headerToo) {
+			preventScrollToTopOnRefresh = true;
+			super.refresh(headerToo);
+		}
+
 	};
 
 }
